/*
 * Copyright (c) 2014 F. Mannhardt (f.mannhardt@tue.nl)
 * 
 * LICENSE:
 * 
 * This code is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 3 of the License, or (at your option) any
 * later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 */
package org.processmining.xeslite.external;

import java.util.AbstractSet;
import java.util.Collection;
import java.util.Iterator;
import java.util.NavigableSet;
import java.util.Set;

import org.deckfour.xes.model.XAttributeMap;

import com.google.common.base.Function;
import com.google.common.collect.Iterators;

/**
 * A {@link Set} implementation that provides a view on the keys of an
 * {@link XAttributeMap}. This implementation queries the underlying MapDB
 * storage and tries to avoid copies where possible. It does not allow for
 * {@link #add(String)}.
 * 
 * @author F. Mannhardt
 * 
 */
final class MapDBKeySet extends AbstractSet<String> {
	
	private final MapDBAttributeMap attributeMap;
	private final StringPool stringPool;
	private final NavigableSet<Long> keySet;

	public MapDBKeySet(MapDBAttributeMap attributeMap) {
		this.attributeMap = attributeMap;
		this.stringPool = attributeMap.getStore().getAttributeKeyPool();
		this.keySet = attributeMap.getInternalSubMap().navigableKeySet();
	}

	@Override
	public Iterator<String> iterator() {
		return Iterators.transform(keySet.iterator(), new Function<Long, String>() {

			@Override
			public String apply(Long id) {
				int keyId = (int) (id.longValue() % stringPool.getPoolLimit());
				return stringPool.getValue(keyId);
			}
		});
	}
	
	@Override
	public int size() {
		return attributeMap.size();
	}

	// Overridden for performance

	@Override
	public void clear() {
		attributeMap.clear();
	}
	
	@Override
	public boolean contains(Object o) {
		return attributeMap.containsKey(o);
	}
	
	@Override
	public boolean remove(Object o) {
		return attributeMap.remove(o) != null;
	}

	@Override
	public boolean removeAll(Collection<?> c) {
		boolean changed = false;
		for (Object object : c) {
			changed = remove(object);
		}
		return changed;
	}

}
