/*
 * Copyright (c) 2013 F. Mannhardt (f.mannhardt@tue.nl)
 * 
 * LICENSE:
 * 
 * This code is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 3 of the License, or (at your option) any
 * later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 */
package org.processmining.xeslite.external;

import java.io.Serializable;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentMap;
import java.util.concurrent.atomic.AtomicReferenceArray;
import java.util.concurrent.locks.ReentrantLock;

final class StringPoolLockImpl implements StringPool, Serializable {

	private static final long serialVersionUID = 4375804650443172521L;

	private final int poolLimit;
	
	private int keyCounter;

	private final ConcurrentMap<String, Integer> keyMap;
	private final AtomicReferenceArray<String> values;

	private final ReentrantLock lock = new ReentrantLock();

	public StringPoolLockImpl(int poolLimit) {
		super();
		this.poolLimit = poolLimit;
		this.keyCounter = 0;
		this.keyMap = new ConcurrentHashMap<>();
		this.values = new AtomicReferenceArray<>(poolLimit);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.progressmining.xeslite.external.StringPool#put(java.lang.String)
	 */
	@Override
	public Integer put(String val) {
		Integer idx = keyMap.get(val);
		if (idx == null) {
			lock.lock();
			try {
				// Bounds check
				if (keyCounter == poolLimit) {
					throw new RuntimeException("Too many distinct keys in this XES file!");
				}
				idx = keyCounter + 1;
				Integer oldIdx = keyMap.putIfAbsent(val, idx);
				if (oldIdx == null) {
					// Insert the reverse mapping
					values.set(idx, val);
					// Update keyCounter, as we succeeded with putting our index 
					keyCounter = idx;
				}
			} finally {
				lock.unlock();
			}
		}
		// Found Index
		return idx;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.progressmining.xeslite.external.StringPool#getIndex(java.lang.String)
	 */
	@Override
	public Integer getIndex(String val) {
		return keyMap.get(val);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.progressmining.xeslite.external.StringPool#getValue(java.lang.Integer
	 * )
	 */
	@Override
	public String getValue(Integer index) {
		return values.get(index);	
	}

	public String toString() {
		return String.format("StringPoolLockImpl [keyCounter=%s, keyMap=%s, values=%s, lock=%s]", keyCounter, keyMap,
				values, lock);
	}

	public int getPoolLimit() {
		return poolLimit;
	}

}